package org.beigesoft.web.factory;

/*
 * Copyright (c) 2017 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;
import java.util.HashMap;

import org.beigesoft.factory.IFactorySimple;
import org.beigesoft.replicator.service.DatabaseReaderIdenticalXml;
import org.beigesoft.replicator.service.SrvClearDatabase;
import org.beigesoft.replicator.service.SrvEntityReaderXml;
import org.beigesoft.replicator.service.ReplicatorXmlHttps;
import org.beigesoft.replicator.service.ISrvEntityFieldFiller;
import org.beigesoft.replicator.service.SrvEntityFieldPersistableBaseRepl;
import org.beigesoft.replicator.service.SrvEntityFieldUserTomcatRepl;
import org.beigesoft.replicator.service.SrvEntityFieldHasIdStringRepl;
import org.beigesoft.replicator.service.SrvEntityFieldHasIdLongRepl;
import org.beigesoft.replicator.service.SrvEntityFieldFillerStd;

/**
 * <p>
 * Simple factory that create a request(or) scoped bean.
 * </p>
 *
 * @author Yury Demidenko
 * @param <RS> platform dependent RDBMS recordset
 **/
public class FactoryReplicatorXmlHttps<RS>
  implements IFactorySimple<ReplicatorXmlHttps<RS>> {

  /**
   * <p>Factory app-beans.</p>
   **/
  private AFactoryAppBeans<RS> factoryAppBeans;

  /**
   * <p>Create a bean.</p>
   * @param pAddParam additional param
   * @return M request(or) scoped bean
   * @throws Exception - an exception
   */
  @Override
  public final ReplicatorXmlHttps<RS> create(
    final Map<String, Object> pAddParam) throws Exception {
    ReplicatorXmlHttps<RS> srvGetDbCopyXml = new ReplicatorXmlHttps<RS>();
    SrvEntityReaderXml srvEntityReaderXml = new SrvEntityReaderXml();
    srvEntityReaderXml.setUtilXml(this.factoryAppBeans.lazyGetUtilXml());
    SrvEntityFieldFillerStd srvEntityFieldFillerStd =
      new SrvEntityFieldFillerStd();
    srvEntityFieldFillerStd.setUtilXml(this.factoryAppBeans.lazyGetUtilXml());
    srvEntityFieldFillerStd.setUtlReflection(this.factoryAppBeans
      .lazyGetUtlReflection());
    SrvEntityFieldPersistableBaseRepl srvEntityFieldPersistableBaseRepl =
      new SrvEntityFieldPersistableBaseRepl();
    srvEntityFieldPersistableBaseRepl
      .setUtlReflection(this.factoryAppBeans.lazyGetUtlReflection());
    SrvEntityFieldHasIdStringRepl srvEntityFieldHasIdStringRepl =
      new SrvEntityFieldHasIdStringRepl();
    srvEntityFieldHasIdStringRepl
      .setUtlReflection(this.factoryAppBeans.lazyGetUtlReflection());
    SrvEntityFieldHasIdLongRepl srvEntityFieldHasIdLongRepl =
      new SrvEntityFieldHasIdLongRepl();
    srvEntityFieldHasIdLongRepl
      .setUtlReflection(this.factoryAppBeans.lazyGetUtlReflection());
    SrvEntityFieldUserTomcatRepl srvEntityFieldUserTomcatRepl =
      new SrvEntityFieldUserTomcatRepl();
    Map<String, ISrvEntityFieldFiller> fieldsFillersMap =
      new HashMap<String, ISrvEntityFieldFiller>();
    fieldsFillersMap.put("SrvEntityFieldFillerStd",
      srvEntityFieldFillerStd);
    fieldsFillersMap.put("SrvEntityFieldPersistableBaseRepl",
      srvEntityFieldPersistableBaseRepl);
    fieldsFillersMap.put("SrvEntityFieldHasIdStringRepl",
      srvEntityFieldHasIdStringRepl);
    fieldsFillersMap.put("SrvEntityFieldHasIdLongRepl",
      srvEntityFieldHasIdLongRepl);
    fieldsFillersMap.put("SrvEntityFieldUserTomcatRepl",
      srvEntityFieldUserTomcatRepl);
    srvEntityReaderXml.setFieldsFillersMap(fieldsFillersMap);
    srvEntityReaderXml.setMngSettings(this.factoryAppBeans
      .lazyGetMngSettingsGetDbCopy());
    DatabaseReaderIdenticalXml<RS> databaseReaderIdenticalXml =
     new DatabaseReaderIdenticalXml<RS>();
    databaseReaderIdenticalXml.setUtilXml(this.factoryAppBeans
      .lazyGetUtilXml());
    databaseReaderIdenticalXml.setSrvEntityReader(srvEntityReaderXml);
    databaseReaderIdenticalXml.setSrvDatabase(this.factoryAppBeans
      .lazyGetSrvDatabase());
    databaseReaderIdenticalXml.setSrvOrm(this.factoryAppBeans.lazyGetSrvOrm());
    databaseReaderIdenticalXml.setLogger(this.factoryAppBeans.lazyGetLogger());
    SrvClearDatabase<RS> srvClearDatabase = new SrvClearDatabase<RS>();
    srvClearDatabase.setMngSettings(this.factoryAppBeans
      .lazyGetMngSettingsGetDbCopy());
    srvClearDatabase.setLogger(this.factoryAppBeans.lazyGetLogger());
    srvClearDatabase.setSrvDatabase(this.factoryAppBeans.lazyGetSrvDatabase());
    srvGetDbCopyXml.setDatabasePrepearerBefore(srvClearDatabase);
    srvGetDbCopyXml.setDatabasePrepearerAfter(this.factoryAppBeans
      .lazyGetPrepareDbAfterFullImport());
    srvGetDbCopyXml.setUtilXml(this.factoryAppBeans.lazyGetUtilXml());
    srvGetDbCopyXml.setSrvEntityReaderXml(srvEntityReaderXml);
    srvGetDbCopyXml.setMngSettings(this.factoryAppBeans
      .lazyGetMngSettingsGetDbCopy());
    srvGetDbCopyXml.setSrvDatabase(this.factoryAppBeans.lazyGetSrvDatabase());
    srvGetDbCopyXml.setDatabaseReader(databaseReaderIdenticalXml);
    srvGetDbCopyXml.setLogger(this.factoryAppBeans.lazyGetLogger());
    return srvGetDbCopyXml;
  }

  // SGS:
  /**
   * <p>Getter for factoryAppBeans.</p>
   * @return AFactoryAppBeans<RS>
   **/
  public final AFactoryAppBeans<RS> getFactoryAppBeans() {
    return this.factoryAppBeans;
  }

  /**
   * <p>Setter for factoryAppBeans.</p>
   * @param pFactoryAppBeans reference
   **/
  public final void setFactoryAppBeans(
    final AFactoryAppBeans<RS> pFactoryAppBeans) {
    this.factoryAppBeans = pFactoryAppBeans;
  }
}
