package org.beigesoft.web.service;

/*
 * Copyright (c) 2017 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

/**
 * <p>Service that check if database has any users credentials, if not
 * it serves creating the first user. This is for localhost only!</p>
 * @author Yury Demidenko
 */
public interface ISrvAddTheFirstUser {

  /**
   * <p>Checks is there any user.</p>
   * @return if there is any user
   * @throws Exception - an exception
   **/
  boolean checkIsThereAnyUser() throws Exception;

  /**
   * <p>Add only user/password/role.</p>
   * @param pUserName User Name
   * @param pPassw User password
   * @param pRole User role
   * @throws Exception - an exception
   **/
  void addUser(String pUserName, String pPassw,
    String pRole) throws Exception;

  /**
   * <p>Change user password.</p>
   * @param pUserName User Name
   * @param pPassw User password
   * @param pPasswOld User password old
   * @return if changed (if there is user with old password)
   * @throws Exception - an exception
   **/
  boolean changeUserPasswd(String pUserName, String pPassw,
    String pPasswOld) throws Exception;

  /**
   * <p>Check if password strong.
   * It implements logic:
   * At least 15 letters and digits!
   * 60% of them must be different!
   * At least 50% of them must be letters!
   * At least 3 of them must be digits!
   * No containing qwerty, 12345, admin, user etc!</p>
   * @param pPassword Password
   * @return NULL if strong, otherwise message.
   **/
  String isPasswordStrong(char[] pPassword);

  /**
   * <p>Getter for isThereAnyUser.</p>
   * @return boolean
   **/
  boolean getIsThereAnyUser();

  /**
   * <p>Setter for isThereAnyUser.</p>
   * @param pIsThereAnyUser reference
   **/
  void setIsThereAnyUser(boolean pIsThereAnyUser);
}
