package org.beigesoft.web.service;

/*
 * Copyright (c) 2017 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.HashSet;
import java.util.ArrayList;

import org.beigesoft.model.IRecordSet;
import org.beigesoft.service.ISrvDatabase;

/**
 * <p>Service that check if database has any users credentials, if not
 * it serves creating the first user and changing his password.
 * This is for localhost only!</p>
 * @param <RS> platform dependent RDBMS recordset
 * @author Yury Demidenko
 */
public class SrvAddTheFirstUser<RS> implements ISrvAddTheFirstUser {

  /**
   * <p>Database service.</p>
   **/
  private ISrvDatabase<RS> srvDatabase;

  /**
   * <p>Is there any user cache.</p>
   **/
  private boolean isThereAnyUser = false;

  /**
   * <p>Checks is there any user.</p>
   * @return if there is any user
   * @throws Exception - an exception
   **/
  @Override
  public final boolean checkIsThereAnyUser() throws Exception {
    if (this.isThereAnyUser) {
      return true;
    }
    String query = "select USERTOMCAT.ITSUSER as ITSUSER, ITSROLE"
      + " from USERTOMCAT join USERROLETOMCAT"
        + " on USERROLETOMCAT.ITSUSER = USERTOMCAT.ITSUSER;";
    boolean result = false;
    IRecordSet<RS> recordSet = null;
    try {
      this.srvDatabase.setIsAutocommit(false);
      this.srvDatabase.
        setTransactionIsolation(ISrvDatabase.TRANSACTION_READ_UNCOMMITTED);
      this.srvDatabase.beginTransaction();
      recordSet = getSrvDatabase().retrieveRecords(query);
      if (recordSet.moveToFirst()) {
        this.isThereAnyUser = true;
        result = true;
      }
      this.srvDatabase.commitTransaction();
    } catch (Exception ex) {
      this.srvDatabase.rollBackTransaction();
      throw ex;
    } finally {
      this.srvDatabase.releaseResources();
      if (recordSet != null) {
        recordSet.close();
      }
    }
    return result;
  }

  /**
   * <p>Add only user/password/role.</p>
   * @param pUserName User Name
   * @param pPassw User password
   * @param pRole User role
   * @throws Exception - an exception
   **/
  @Override
  public final void addUser(final String pUserName,
    final String pPassw, final String pRole) throws Exception {
    String query = "select USERTOMCAT.ITSUSER as ITSUSER, ITSROLE"
      + " from USERTOMCAT join USERROLETOMCAT"
        + " on USERROLETOMCAT.ITSUSER = USERTOMCAT.ITSUSER;";
    IRecordSet<RS> recordSet = null;
    try {
      this.srvDatabase.setIsAutocommit(false);
      this.srvDatabase.
        setTransactionIsolation(ISrvDatabase.TRANSACTION_READ_UNCOMMITTED);
      this.srvDatabase.beginTransaction();
      recordSet = getSrvDatabase().retrieveRecords(query);
      if (recordSet.moveToFirst()) {
        throw new Exception("There are already user credentials!");
      }
      String addUserQ =
        "insert into USERTOMCAT (ITSUSER, ITSPASSWORD) values ('" + pUserName
          + "', '" + pPassw + "');";
      String addUserRoleQ =
        "insert into USERROLETOMCAT (ITSUSER, ITSROLE) values ('" + pUserName
          + "', '" + pRole + "');";
      this.srvDatabase.executeQuery(addUserQ);
      this.srvDatabase.executeQuery(addUserRoleQ);
      this.srvDatabase.commitTransaction();
      this.isThereAnyUser = true;
    } catch (Exception ex) {
      this.srvDatabase.rollBackTransaction();
      throw ex;
    } finally {
      this.srvDatabase.releaseResources();
      if (recordSet != null) {
        recordSet.close();
      }
    }
  }


  /**
   * <p>Change user password.</p>
   * @param pUserName User Name
   * @param pPassw User password
   * @param pPasswOld User password old
   * @return if changed (if there is user with old password)
   * @throws Exception - an exception
   **/
  @Override
  public final boolean changeUserPasswd(final String pUserName,
    final String pPassw, final String pPasswOld) throws Exception {
    String query = "select USERTOMCAT.ITSUSER from USERTOMCAT where ITSUSER='"
      + pUserName + "' and ITSPASSWORD='" + pPasswOld + "';";
    IRecordSet<RS> recordSet = null;
    try {
      this.srvDatabase.setIsAutocommit(false);
      this.srvDatabase.
        setTransactionIsolation(ISrvDatabase.TRANSACTION_READ_UNCOMMITTED);
      this.srvDatabase.beginTransaction();
      recordSet = getSrvDatabase().retrieveRecords(query);
      if (!recordSet.moveToFirst()) {
        return false;
      }
      String changeUserP =
        "update USERTOMCAT set ITSPASSWORD='" + pPassw + "' where ITSUSER='"
          + pUserName + "';";
      this.srvDatabase.executeQuery(changeUserP);
      this.srvDatabase.commitTransaction();
      return true;
    } catch (Exception ex) {
      this.srvDatabase.rollBackTransaction();
      throw ex;
    } finally {
      this.srvDatabase.releaseResources();
      if (recordSet != null) {
        recordSet.close();
      }
    }
  }

  /**
   * <p>Check if password strong.
   * It implements logic:
   * At least 15 letters and digits!
   * 60% of them must be different!
   * At least 50% of them must be letters!
   * At least 3 of them must be digits!
   * No containing qwerty, 12345, admin, user etc!</p>
   * @param pPassword Password
   * @return NULL if strong, otherwise message.
   **/
  @Override
  public final String isPasswordStrong(final char[] pPassword) {
    if (pPassword == null || pPassword.length < 15) {
      return "Password15";
    }
    String passw = new String(pPassword).toLowerCase();
    if (passw.contains("qwert") || passw.contains("qwaszx")
      || passw.contains("qweasd") || passw.contains("qazwsx")
        || passw.contains("wsxedc") || passw.contains("wqsaxz")
          || passw.contains("ewqdsa") || passw.contains("zaqxsw")
            || passw.contains("xswzaq") || passw.contains("qscwdv")
              || passw.contains("csqvdw") || passw.contains("zaxqsc")
                || passw.contains("qscax") || passw.contains("csqxa")
                  || passw.contains("trewq") || passw.contains("asdfg")
                || passw.contains("zxcvb") || passw.contains("bvcxz")
              || passw.contains("gfdsa")) {
      return "noQwerty";
    } else if (passw.contains("raccooneatstone")
      || passw.contains("nraccooteaeston")) {
      return "noDemoPassw";
    } else if (passw.contains("2345") || passw.contains("admin")
      || passw.contains("user") || passw.contains("5432")
        || passw.contains("5678") || passw.contains("9876")
          || passw.contains("password")) {
      return "noAdmin12345";
    }
    HashSet<Character> chars = new HashSet<Character>();
    ArrayList<Character> digits = new ArrayList<Character>();
    ArrayList<Character> letters = new ArrayList<Character>();
    for (char ch : pPassword) {
      if (!Character.isLetterOrDigit(ch)) {
        return "letterOrDig";
      }
      if (Character.isDigit(ch)) {
        digits.add(ch);
      } else {
        letters.add(ch);
      }
      chars.add(ch);
    }
    double allLn = pPassword.length;
    double lettersLn = letters.size();
    double distinctLn = chars.size();
    if (lettersLn / allLn < 0.49999999999) {
      return "lettersAtLeast50pr";
    }
    if (distinctLn / allLn < 0.59999999999) {
      return "distinct60pr";
    }
    if (digits.size() < 3) {
      return "atLeast3digits";
    }
    return null;
  }

  /**
   * <p>Getter for isThereAnyUser.</p>
   * @return boolean
   **/
  @Override
  public final boolean getIsThereAnyUser() {
    return this.isThereAnyUser;
  }

  /**
   * <p>Setter for isThereAnyUser.</p>
   * @param pIsThereAnyUser reference
   **/
  @Override
  public final void setIsThereAnyUser(final boolean pIsThereAnyUser) {
    this.isThereAnyUser = pIsThereAnyUser;
  }

  /**
   * <p>Geter for srvDatabase.</p>
   * @return ISrvDatabase
   **/
  public final ISrvDatabase<RS> getSrvDatabase() {
    return this.srvDatabase;
  }

  /**
   * <p>Setter for srvDatabase.</p>
   * @param pSrvDatabase reference
   **/
  public final void setSrvDatabase(final ISrvDatabase<RS> pSrvDatabase) {
    this.srvDatabase = pSrvDatabase;
  }
}
